/*
 * Kylin security modules
 *
 * Author: penghuaihe <penghuaihe@kylinos.cn>
 *
 * Copyright (c) KylinSoft Co., Ltd. 2024.All rights reserved.
 */

#ifndef _KSAF_APP_H_
#define _KSAF_APP_H_

#ifndef _KSAF_FRAMEWORK_H_
typedef union {
	struct {
		uint64_t pkg_id;
		uint64_t file_id;
		uint64_t path_id;
	};
	uint64_t id;
} KSAF_SID;
#endif

/* 此文件内的定义，与上层协商保持一致，禁止随意修改， 必须与上层评估修改影响 */

/*
 * 如下是与标签管理工具协商的相关数据结构
 */

#define KSAF_SUBJECT   1       /* 主体 */
#define KSAF_OBJECT    2       /* 客体 */

/* 进程身份转换标识 */
#define TRANS_TYPE_INTERP_BASH	BIT(0)
#define TRANS_TYPE_APP		BIT(1)
#define TRANS_TYPE_OVERLAYFS	BIT(2) /* overlayfs要进行放权，通过此特定标志实现 */

#define KSAF_ID_PAIR_MSG_TYPE_ADD	1
#define KSAF_ID_PAIR_MSG_TYPE_DEL	2
#define KSAF_ID_PAIR_MSG_TYPE_CLR	3

struct id_pairs_header {
	uint32_t len_sum;
	uint32_t pair_count;
	uint32_t msg_type;
	uint32_t path_id_type;
} __attribute__((aligned(4)));

struct ksaf_id_pair {
	uint64_t current_id;
	uint64_t parent_id;
}__attribute__((aligned(4)));

struct ksaf_uuid_rule {
	unsigned long inode;
	uint64_t file_id;
	unsigned char uuid[16];
}__attribute__((aligned(4)));

#define KSAF_LABEL_CREATE	1
#define KSAF_LABEL_RENAME	2

struct obj_label_msg_response {
	int	serial;
	int is_pkg_manager;
}__attribute__((aligned(4)));

struct obj_label_msg_user {
	int	serial;
	int flag;
	pid_t pid;
}__attribute__((aligned(4)));

/*
 * 如下是与策略编译器协商的相关数据结构
 */
struct namespace_func_node {
	uint32_t func_id;
	uint32_t func_len;
} __attribute__((aligned(4)));

struct namespace_node {
	uint32_t namespace_id;
	uint32_t name_len;
	uint32_t func_count;
	struct namespace_func_node *head;
} __attribute__((aligned(4)));


struct policy_stream_header {
	uint32_t version;
	uint32_t len_sum;
	uint32_t len_per_policy;
	uint32_t policy_num;
	uint32_t msg_type;
} __attribute__((aligned(4)));

struct expr_para_info {
	uint32_t para_type;
	uint32_t para_len;
} __attribute__((aligned(4)));

struct expr_node_user {
	uint32_t expr_type;
	uint32_t expr_value;
	uint32_t func_id;
	uint32_t para_num;
	uint32_t name_len;
} __attribute__((aligned(4)));

/* 使用bit位表示， 便于查询结果返回多个 */
#define OP_TYPE_ALLOW		1
#define OP_TYPE_DISALLOW	2
#define OP_TYPE_DETECT		4
#define OP_TYPE_ALLOW_ONCE	8
#define OP_TYPE_RESPOND		16

/** 策略类型， 对应 struct ksaf_policy_user->policy_type成员 */
#define POLICY_TYPE_ACCURATE	1
#define POLICY_TYPE_WILDCARD	2
#define POLICY_TYPE_MAX		3

#define GRP_PARA_FLAG		0
#define BIT_AUDIT		BIT(1)
#define BIT_PERMISSIVE		BIT(2)
#define BIT_INTERACTIVE		(BIT(3) | BIT(4))
#define INTERACTIVE_MODE_NONE	0
#define INTERACTIVE_MODE_AUTH	BIT(3)
#define INTERACTIVE_MODE_NOTIFY	BIT(4)

#define GRP_PARA_NS_ID		1
#define GRP_PARA_PRIORITY	2
#define GRP_PARA_NUM		5

struct ksaf_policy_user {
	uint32_t policy_type;

	uint32_t uid;
	KSAF_SID sid;
	KSAF_SID oid;
	uint32_t op_type;
	uint32_t perm_class_id;
	uint32_t perm_id;

	uint32_t group_para[GRP_PARA_NUM];

	uint32_t exp_count;
	uint32_t do_func_count;
} __attribute__((aligned(4)));

#endif
