/*
 * Kylin security modules
 *
 * Author: penghuaihe <penghuaihe@kylinos.cn>
 *
 * Copyright (c) KylinSoft Co., Ltd. 2024.All rights reserved.
 */

#ifndef _KSAF_PUBLIC_H_
#define _KSAF_PUBLIC_H_

/**
 * 此文件用与存放内核与上层应用协商的一些数据结构体或定义。请勿随意修改。 
 */

#include <linux/ksaf_framework.h>
#include <linux/audit.h>
#include <linux/ksaf_devctl.h>
#include <vdso/bits.h>
#include <uapi/linux/magic.h>

#include "ksaf_app.h"

#define KSAF_STATUS_DISABLED	0
#define KSAF_STATUS_ENFORCE	2
#define KSAF_STATUS_PERMISSIVE	4

#define XATTR_KSAF_SUFFIX "ksaf"
#define XATTR_NAME_KSAF XATTR_SECURITY_PREFIX XATTR_KSAF_SUFFIX

#define KSAF_UINT64_HIGH	0xffffffff00000000

#define KSAF_CHAR_MID_LEN 128

/*
 * 客体类别， 对应 struct ksaf_policy_user->perm_class_id 成员,对客体类别(比如file)
 * 或者某个客体下操作(比如file的wirite)进行扩充的时候，需要同步修改 class-def.conf 文件,
 * 因为策略编译器需要解析这个文件，才能正确识别出客体类型。某些情况下，扩充了客体类别，还需要
 * 策略编译器同步修改，所以，修改之前最好跟策略编译器对齐一下，内核模块能否单方面扩充客体类别。
 */
#define KSAF_OBJ_CLASS_COMMON		0
#define KSAF_OBJ_CLASS_FILE		1
#define KSAF_OBJ_CLASS_TASK		2
#define KSAF_OBJ_CLASS_KMODULE		3
#define KSAF_OBJ_CLASS_SOCKET		5
#define KSAF_OBJ_CLASS_CAP		6
#define KSAF_OBJ_CLASS_LINKFILE		7
#define KSAF_OBJ_CLASS_DIR		8

#define KSAF_OBJ_CLASS_DEVICE_BASE		20
#define	KSAF_OBJ_CLASS_DEVICE_USB		21
#define	KSAF_OBJ_CLASS_DEVICE_PCI		22
#define	KSAF_OBJ_CLASS_DEVICE_SERIAL		23
#define	KSAF_OBJ_CLASS_DEVICE_PARALLEL		24
#define	KSAF_OBJ_CLASS_DEVICE_IEEE1394		25
#define	KSAF_OBJ_CLASS_DEVICE_PCMCIA		26
#define	KSAF_OBJ_CLASS_DEVICE_PCIE		27
#define	KSAF_OBJ_CLASS_DEVICE_SATA		28
#define	KSAF_OBJ_CLASS_DEVICE_HDMI		29
#define	KSAF_OBJ_CLASS_DEVICE_DP		30
#define	KSAF_OBJ_CLASS_DEVICE_CDROM		31
#define	KSAF_OBJ_CLASS_DEVICE_PRINTER		32
#define	KSAF_OBJ_CLASS_DEVICE_WIRELESS		33
#define	KSAF_OBJ_CLASS_DEVICE_ETHERNET		34
#define	KSAF_OBJ_CLASS_DEVICE_BLUETOOTH		35
#define	KSAF_OBJ_CLASS_DEVICE_VEDIO		36
#define	KSAF_OBJ_CLASS_DEVICE_AD_JACK		37
#define	KSAF_OBJ_CLASS_DEVICE_AD_MIC		38
#define	KSAF_OBJ_CLASS_DEVICE_AD_SPK		39
#define	KSAF_OBJ_CLASS_DEVICE_INFRARED		40
#define	KSAF_OBJ_CLASS_DEVICE_MODEM		41
#define	KSAF_OBJ_CLASS_DEVICE_MMC		42
#define	KSAF_OBJ_CLASS_DEVICE_SCSI_RAID		43
#define	KSAF_OBJ_CLASS_DEVICE_FLPY_DRV		44
#define	KSAF_OBJ_CLASS_DEVICE_MUL_SERIAL_CARD	45
#define	KSAF_OBJ_CLASS_DEVICE_HOT_SPOT		46
#define	KSAF_OBJ_CLASS_DEVICE_SCANNER		47
#define	KSAF_OBJ_CLASS_DEVICE_WINCE		48
#define	KSAF_OBJ_CLASS_DEVICE_PRINTER_LOCAL	49
#define	KSAF_OBJ_CLASS_DEVICE_PRINTER_NET	50
#define	KSAF_OBJ_CLASS_DEVICE_TYPEC		51
#define	KSAF_OBJ_CLASS_DEVICE_VOLUME_SHADOW	52
#define	KSAF_OBJ_CLASS_DEVICE_TAPE		53
#define	KSAF_OBJ_CLASS_DEVICE_PS2		54
#define	KSAF_OBJ_CLASS_DEVICE_STORAGE_DRIVE	55
#define KSAF_OBJ_CLASS_DEVICE_MAX	KSAF_OBJ_CLASS_DEVICE_BASE + KSAF_DEV_MAX
#define KSAF_OBJ_CLASS_MAX		64

/** 文件操作集合，对应 struct ksaf_policy_user->perm_id 成员  */
#define KSAF_FILE_OPEN		1
#define KSAF_FILE_READ		2
#define KSAF_FILE_WRITE		3
#define KSAF_FILE_TRUNCATE	4
#define KSAF_FILE_EXEC		5
#define KSAF_FILE_MKDIR		6
#define KSAF_FILE_RMDIR		7
#define KSAF_FILE_LINK		8
#define KSAF_FILE_UNLINK	9
#define KSAF_FILE_SYMLINK	10
#define KSAF_FILE_RENAME	11
#define KSAF_FILE_SETATTR	12
#define KSAF_FILE_GETATTR	13
#define KSAF_FILE_SETXATTR	14
#define KSAF_FILE_GETXATTR	15
#define KSAF_FILE_KMOD_INSERT	16
#define KSAF_FILE_SHOW		17
#define KSAF_FILE_INTERPRET	18
#define KSAF_FILE_EXEC_TMPFS	19
#define KSAF_FILE_SETKSAF	20
#define KSAF_FILE_INVALID	21

/** 设备操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_DEVICE_USE	1
#define KSAF_DEVICE_WRITE	3

/** 进程操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_TASK_KILL		1

/** 内核模块操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_KMOD_REMOVE	1

/** Socket操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_SOCKET_CONNECT	1

#define KSAF_CAP_USE		1

#define KSAF_TRUE	1
#define KSAF_FALSE	0

#define KSAF_UINT32_MAX 0xffffffff
#define KSAF_UINT64_MAX 0xffffffffffffffff

struct expr_func_para {
	uint64_t d_64;
	unsigned char *hash;
	uint32_t hash_len;
	struct expr_func_para *next;
}__attribute__((aligned(4)));

/* 存储动态变量相关的文件，包括namespace文件夹以及其下的变量 */
struct ksaf_dynamic_para {
	struct list_head list;
	atomic_t ref_cnt;

	struct ksaf_dynamic_para *parent;
	struct dentry *dentry;
	char *ns_name;
	char *para_name;
	uint32_t value;
};

struct expr_node_kern {
	atomic_t ref;
	struct expr_node_user user;
	struct expr_func_para *para;
	struct expr_node_kern *left;
	struct expr_node_kern *right;
	struct ksaf_dynamic_para *d_para;
};

struct ksaf_policy_kern {
	struct list_head list;
	struct rcu_head rcu;
	struct ksaf_policy_user policy;
	struct expr_node_kern *root;
	struct expr_node_kern *do_func;
};

struct ksaf_filectl_para {
	struct linux_binprm *bprm;
	struct file *file;
	struct inode *inode;
	struct ksaf_common_inode_security *isec;
	struct ksaf_common_task_security *tsec;
	u64 ino;
	uid_t task_euid;
};

struct ksaf_devctl_para {
	struct device *dev;
	int type;
	unsigned int pid;
	unsigned int vid;
	unsigned int if_protocol;
	char product[KSAF_CHAR_MID_LEN];
	char manufacturer[KSAF_CHAR_MID_LEN];
	char serial[KSAF_CHAR_MID_LEN];
	char name[KSAF_CHAR_MID_LEN];
};

struct ksaf_kmodpro_para {
	char *modname;
};

struct ksaf_taskpro_para {
	struct task_struct *task;
	KSAF_SID obj_orig_sid;
	struct ksaf_common_task_security *stsec;
	struct ksaf_common_task_security *otsec;
};

struct ksaf_cap_para {
	const struct cred *cred;
	int cap;
};

/**
 * 文件管控、设备管控、模块防卸载、进程防杀等管控模块的钩子，所传递的参数类型
 * 是不一样的，但是，框架中又需要提供统一的表达式函数注册接口，并且要尽量保证
 * 内核代码的稳定、精简。 因此，表达式函数ksaf_add_funcs[KSAF_ADDFUNC_SUPP_MAX]
 * 采用void *类型。各管控模块根据自己需要的传参类型，使用此联合体的对应成员即可，
 * 在表达式函数中也做同样的类型转换即可。
 */
struct ksaf_funcs_para {
	uint32_t uid;
	KSAF_SID sid;
	KSAF_SID oid;
	KSAF_SID orig_sid;
	uint32_t perm_class_id;
	uint32_t perm_id;

	struct ksaf_policy_kern *policy;
	struct expr_node_kern *expr;
	struct ksaf_policy_kern *policy_final;

	union {
		struct ksaf_filectl_para file_para;
		struct ksaf_devctl_para dev_para;
		struct ksaf_kmodpro_para kmodpro_para;
		struct ksaf_taskpro_para taskpro_para;
		struct ksaf_cap_para cap_para;
	} para;
};

/** 扩展模块用于定义if表达式函数的结构 */
struct ksaf_add_func_member {
	uint32_t (*func)(void *);
	char *desc;
};

struct ksaf_list_head {
	struct list_head list;
	spinlock_t lock;
};

struct ksaf_list_head_m {
	struct list_head list;
	struct mutex lock;
};

#ifdef CONFIG_AUDIT
static inline kuid_t ksaf_get_loginuid(void)
{
	kuid_t uid;

	uid = audit_get_loginuid(current);
	if (uid.val == KSAF_UINT32_MAX)
		return current_euid();
	return uid;
}
#else
static inline kuid_t ksaf_get_loginuid(void)
{
	return current_euid();
}
#endif

extern int ksaf_common_policy_query(void *data, bool audit);
extern int ksaf_common_policy_stream_parse(void *data, size_t datalen);
extern uint64_t ksaf_common_path_id_pair_query(uint64_t current_id, uint32_t id_type);

extern void ksaf_policy_notify_update(void);
extern void ksaf_common_policydb_updated(void);

extern struct dentry *ovl_dentry_real(struct dentry *dentry);
extern struct inode *ovl_inode_real(struct inode *inode);

extern uint32_t ksaf_status;
static inline uint32_t ksaf_get_status(void)
{
	return ksaf_status;
}

static inline bool ksaf_sid_equal(KSAF_SID *sid_l, KSAF_SID *sid_r)
{
	if (sid_l->pkg_id == sid_r->pkg_id && sid_l->file_id == sid_r->file_id &&
	    sid_l->path_id == sid_r->path_id)
		return true;

	return false;
}

static void inline ksaf_string_ncpy(char *dest, char *src, int count)
{
	while (count--) {
		if ((*dest++ = *src++) == '\0')
			break;
	}
}

static int inline ksaf_string_ncpy_ret(char *dest, char *src, int count)
{
	int i = 0;

	while (count--) {
		if ((*dest++ = *src++) == '\0')
			break;

		i++;
	}

	return i;
}

static int inline ksaf_string_len(const char *s)
{
	const char *sc;

	for (sc = s; *sc != '\0'; ++sc)
		/* nothing */;

	return sc - s;
}

static int inline ksaf_string_cmp(const char *cs, const char *ct)
{
	unsigned char c1, c2;

	while (1) {
		c1 = *cs++;
		c2 = *ct++;
		if (c1 != c2)
			return c1 < c2 ? -1 : 1;
		if (!c1)
			break;
	}
	return 0;
}

static inline bool ksaf_d_is_overlay(struct dentry *dentry)
{
	if (dentry->d_sb && dentry->d_sb->s_magic == OVERLAYFS_SUPER_MAGIC)
		return true;

	return false;
}

static inline struct dentry *ksaf_real_dentry(struct dentry *dentry)
{
#if IS_ENABLED(CONFIG_OVERLAY_FS)
	struct dentry *real_dentry;

	if (unlikely(!dentry))
		return NULL;

	if (ksaf_d_is_overlay(dentry)) {
		real_dentry = ovl_dentry_real(dentry);
		if (unlikely(!real_dentry))
			real_dentry = dentry;
	} else {
		real_dentry = dentry;
	}

	return real_dentry;
#else
	return dentry;
#endif
}

static inline bool ksaf_i_is_overlay(struct inode *inode)
{
	if (inode->i_sb && inode->i_sb->s_magic == OVERLAYFS_SUPER_MAGIC)
		return true;

	return false;
}

static inline struct inode *ksaf_real_inode(struct inode *inode)
{
#if IS_ENABLED(CONFIG_OVERLAY_FS)
	struct inode *real_inode;

	if (unlikely(!inode))
		return NULL;

	if (ksaf_i_is_overlay(inode)) {
		real_inode = ovl_inode_real(inode);
		if (unlikely(!real_inode))
			real_inode = inode;
	} else {
		real_inode = inode;
	}

	return real_inode;
#else
	return inode;
#endif
}

extern struct dentry *ksaf_securityfs_dir;

extern int ksaf_add_function_register(uint32_t (*func)(void *));
extern int ksaf_add_function_unregister(uint32_t (*func)(void *));

extern char *ksaf_get_real_filepath_by_inode(struct inode *inode);
extern char *ksaf_get_real_filepath_by_dentry(struct dentry *dentry);
extern char *ksaf_get_real_filepath_by_task(struct task_struct *task);

#endif
